% FLC Servo
% Refer Review Example 12.1
clc; close all; clear all
% Create new FIS with filename "FISservo.fis"
sys=newfis('FLCservo')
% Define membership functions for the
% Input variable "Rate of change of error"
sys=addvar(sys,'input','VELOCITY',[-1 1]);
sys=addmf(sys,'input',1,'N','trimf',[-1 -1 1]);
sys=addmf(sys,'input',1,'P','trimf',[-1 1 1]);
% Define membership functions for the input variable "Error"
sys=addvar(sys,'input','ERROR',[-1 1]);
sys=addmf(sys,'input',2,'N','trimf',[-1 -1 1]);
sys=addmf(sys,'input',2,'P','trimf',[-1 1 1]);
% Define membership functions for the 
% Output variable "Incremental change in input to the motor"
sys=addvar(sys,'output','CONTROL',[-1 1]);
sys=addmf(sys,'output',1,'N','trimf',[-1 -1 0]);
sys=addmf(sys,'output',1,'Z','trimf',[-1 0 1]);
sys=addmf(sys,'output',1,'P','trimf',[0 1 1]);
% Define fuzzy rules
rulelist=[2 2 3 1 1;1 2 2 1 1;2 1 2 1 1;1 1 1 1 1];
sys=addrule(sys,rulelist);
% Define gain constants
GE=0.03;
GV=0.05;
GU=60;
% Define reference signal
for k=1:500
    if (k<=250)
        yref(k)=20;
    else
        yref(k)=23;
    end
end
% Command line simulation
y(1)=0; y(2)=0; u(1)=0;
for k=1:499
    e(k)=yref(k)-y(k);
    if(k==1)
        v(1)=0;
    else
        v(k)=(e(k)-e(k-1))/0.25;
    end
    einp(k)=e(k)*GE;
    vinp(k)=v(k)*GV;
    delu(k)=evalfis([vinp(k) einp(k)],sys);
    u(k+1)=u(k)+delu(k)*GU;
    if(k==1)
        y(2)=0;
    else
        y(k+1)=0.0237*u(k)+0.0175*u(k-1)+1.407*y(k)-0.407*y(k-1);
    end    
end
k=1:500;
plot(k,yref,'linewidth',3,'color',[0 1 0]);
grid
axis([0 500 0 26]);
xlabel('temparature in degree');ylabel('yref in green & y in red');
hold on
plot(k,y,'linewidth',3,'color',[1 0 0]);
% The fuzzy system created entirely from command line may be 
% embedded directly into Simulink to test it out in a simulation
% environment
% The FIS structure for this problem has been created in FIS editor and
% saved as FuzzyPB15.fis
% Open the file FuzzyPB15 in FIS editor and export to workspace 
% (menu driven from toolbar)
% Open SimulinkFigPB15 and execute