% FLC Process
clc; close all; clear all;

global alpha beta y0 T F g sys yref simtim
% Parameters of the plant model 
simtim=360;
alpha=1.00151e-4;
beta=8.67973e-3;
y0=25; y(1)=y0; u(1)=0;
T=30;
F=exp(-alpha*T);
g=(beta/alpha)*(1-F);
% Create a new FIS  with filename "FLCprocess.fis"
sys=newfis('FLCprocess')
% Build the I/O membership functions and fuzzy rules
% Define membership functions for the 
% Input variable "Rate of change of error"
sys=addvar(sys,'input','VELOCITY',[-1 1]);
sys=addmf(sys,'input',1,'N','trimf',[-1 -1 0]);
sys=addmf(sys,'input',1,'NZ','trimf',[-1 -0.1 0.1 1]);
sys=addmf(sys,'input',1,'P','trimf',[0 1 1]);
% Define membership functions for the input variable "error"
sys=addvar(sys,'input','ERROR',[-1 1]);
sys=addmf(sys,'input',2,'N','trimf',[-1 -1 0]);
sys=addmf(sys,'input',2,'NZ','trimf',[-1 -0.1 0.1 1]);
sys=addmf(sys,'input',2,'P','trimf',[0 1 1]);
% Define membership functions for the output variable 
% "Incremental change in input to the thyristor"
sys=addvar(sys,'output','CONTROL',[-1 1]);
sys=addmf(sys,'output',1,'N','trimf',[-1 -1 0]);
sys=addmf(sys,'output',1,'Z','trimf',[-1 0 1]);
sys=addmf(sys,'output',1,'P','trimf',[0 1 1]);
% Define fuzzy rules
rulelist=[1 1 1 1 1;1 2 1 1 1;1 3 2 1 1;2 1 1 1 1;...
    2 2 2 1 1;2 3 3 1 1;3 1 2 1 1;3 2 3 1 1;3 3 3 1 1];
sys=addrule(sys,rulelist);


% Define reference signal
yref=ones(1,simtim)*35;


% Using Genetic Algorithm, obtain optimum values of gain constants
% Create MATLAB function 'objprocessPB16b' before executing function 'ga'

global N nbit minmax itr
itr=0;
N=3;  % Number of variables
nbit=[20 20 40]; % Number of bits for each variable
minmax=[0 1;0 1;0 500]; % Minimum and maximum value for each variable

FitnessFunction = @objprocessPB16b;
numberOfVariables = 80; % Number of bits is 20+20+40 = 80
options = gaoptimset('populationsize',50,'Populationtype','bitstring',...
    'Generations',10,'StallGenLimit',inf,'StallTimeLimit',inf);
[x,fval] = ga(FitnessFunction,numberOfVariables,options);

% Decoding the output of ga (which is binary) to decimal
b1=0;
bn=0;
for j=1:N
    sum1=0;
    n=nbit(j);
    if (j==1)
        b1=1;
    else
        b1=b1+nbit(j-1);
    end
    bn=bn+nbit(j);
    % Calculate the decimal value:
    for k=b1:bn
        n=n-1;
        sum1=sum1+x(k)*(2^n);
    end
    var(j)=sum1;
end

% Scaling between min and max values
for j=1:N
    param(j)=minmax(j,1)...
        +(minmax(j,2)-minmax(j,1))/...
        (2^nbit(j)-1)*var(j);
end



% Define gain constants
GE=param(1); GV=param(2); GU=param(3);

% Simulation results
for k=1:(simtim-1)
    e(k)=yref(k)-y(k);
    if(k==1)
        v(k)=0;
    else
        v(k)=(e(k)-e(k-1))/T;
    end
    ee(k)=e(k)*GE;
    vv(k)=v(k)*GV;
    if(ee(k)>=-1 & ee(k)<=1 & vv(k)>=-1 & vv(k)<=1)
        delu(k)=evalfis([vv(k) ee(k)],sys)*GU;
    else
        delu(k)=0; 
    end    
    u(k+1)=u(k)+delu(k);    
    if(u(k+1)>5)
        u(k+1)=5;
    elseif(u(k+1)<0)
        u(k+1)=0;
    end    
    y(k+1)=F*y(k)+(g*u(k+1)/(1+exp(0.5*y(k)-40)))+(1-F)*y0;
end
k=1:simtim;
plot(k,yref,k,y,'r');