% Function approximation using anfis
clc; clear all; close all;

% The given function y=f(x1,x2,x3)
% Training data: input pattern and output pattern
% x1 x2 and x3 spaced between 1 and 6 with 6 points
traindata=[];
for x1=1:6
    for x2=1:6
        for x3=1:6
            tmp=(1+x1^0.5+x2^(-1)+x3^(-1.5))^2;
            traindata=[traindata;[x1 x2 x3 tmp]];
        end
    end
end

% Create an initial fis
nmf=2; % Number of membership functions
% genfis1 is used to generate sugeno type fis
fis_initial = genfis1(traindata,nmf);

% Train the fis for optimal membership function parameters
epochs=100;
ss=0.1;
[fis_trained train_error step_size] = ...
	anfis(traindata, fis_initial, [epochs nan ss]);

% Testing data: input pattern and output pattern
% x1 x2 and x3 spaced between 1.5 and 5.5 with 5 points
testdata=[];
for x1=1.5:5.5
    for x2=1.5:5.5
        for x3=1.5:5.5
            tmp=(1+x1^0.5+x2^(-1)+x3^(-1.5))^2;
            testdata=[testdata;[x1 x2 x3 tmp]];
        end
    end
end

% Output of trained anfis for new input testdata
yycap = evalfis([testdata(:,1),testdata(:,2),testdata(:,3)],fis_trained);
% Testing-input plot
figure(1);
np=size(yycap,1);
plot(1:np,testdata(:,4),1:np,yycap,'r');
% Plot of error
figure(2)
plot(1:epochs,train_error);

% Plot of initial membership functions
figure(3)
subplot(221); plotmf(fis_initial,'input',1);
subplot(222); plotmf(fis_initial,'input',2);
subplot(223); plotmf(fis_initial,'input',3);
% Plot of trained membership functions
figure(4)
subplot(221); plotmf(fis_trained,'input',1);
subplot(222); plotmf(fis_trained,'input',2);
subplot(223); plotmf(fis_trained,'input',3);