% ANFIS process control
clc; clear all; close all;
% Process model 
T=30; Y0=25;
alpha=1.00151e-4; beta=8.67973e-3;
F=exp(-alpha*T); g=beta*(1-F)/alpha;
y(1)=0;

% Generation of training data
% Input is a train of pulses of random height between 
% 0 and 10 & random width between 0 and 10
s=0;
for j=1:10
    width(j)=rand*10;
    height(j)=rand*10;  
    for k=1:10
        if ( k<=width(j))
            u(k+s)=height(j);
        else
            u(k+s)=0;
        end
        y(k+s+1)=F*y(k+s)+g*u(k+s)/(1+exp(0.5*y(k+s)-40))+(1-F)*Y0;
    end
    s=s+10;
end
NNinput=[y(2:101);y(1:100)];
NNoutput=[u(1:100)];

% Preprocessing of data
NNinput=NNinput/80;
NNoutput=NNoutput/5;

% Training data for ANFIS
train_data=[NNinput' NNoutput'];

% Create an initial fis
nmf=2; % Number of membership functions
% genfis1 is used to generate sugeno type fis
fis_initial = genfis1(train_data,nmf);

% Train the fis for optimal membership function parameters
epochs=100;
ss = 0.1;
[fis_trained train_error step_size] = ...
	anfis(train_data, fis_initial, [epochs nan ss]);

% Plot of initial membership functions
figure(2)
subplot(211); plotmf(fis_initial,'input',1);
subplot(212); plotmf(fis_initial,'input',2);

% Membership functions of trained FIS
figure(3)
subplot(211); plotmf(fis_trained,'input',1);
subplot(212); plotmf(fis_trained,'input',2);


% Output of trained ANFIS
ucap = evalfis(NNinput',fis_trained);
ucap=ucap*5;
figure(1)
k=1:100;
plot(k,ucap,'r',k,u);

% Reference trajectory
for i=1:90
    if (i<=30)
        yr(i)=35;
    elseif (i<=60)
        yr(i)=55;
    else
        yr(i)=75;
    end
end

% Command line simulation
y_sim(1)=Y0;
for k=1:89
    inp=[yr(k+1);y_sim(k)]/80;
    u_sim(k)=evalfis(inp',fis_trained)*5;
    y_sim(k+1)=F*y_sim(k)+g*u_sim(k)...
    /(1+exp(0.5*y_sim(k)-40))+(1-F)*Y0;
end
k=1:90;
figure(4)
plot(k,yr,k,y_sim,'r')
% After execution of this program, the trained FIS structure fis_trained is
% available in workspace
% This is required for execution of simulink file.
% Open SimulinkFigPB19 and execute