% Adaptive Identification
clear all; clc; close all;
% Input signal
N=100; T=10;
index=1:N;
u=0.6*sin(2*pi*index/T)+1.2*cos(2*pi*index/T);
% Measured values of the output
y(1)=u(1);y(2)=u(2)+0.5*u(1);

for k=3:N
    y(k)=u(k)+0.5*u(k-1)-1.5*u(k-2);
end
randn('state',0);
ym=y+0.1*randn(size(y));

% Training data
for k1=3:N
    k=k1-2;
    P(1,k)=u(k1);
    P(2,k)=u(k1-1);
    P(3,k)=u(k1-2);
    t(k)=ym(k1);
end

% Initial NN
a=min(u); b=max(u);
net=newff([a b;a b;a b],[20 1],{'tansig','purelin'},'traingdm');

% Incremental training

net.trainparam.epochs=200;
net.trainparam.goal=1.0e-4;
net.trainparam.lr=0.5;

% Convert the input and output from numeric array
% to cell array for incremental training

P1=num2cell(P,1);
t1=num2cell(t,1);
disp('starting adaptation. please be patient . . .');

for i=1:10
    [net,netout,neterror]=adapt(net,P1,t1);
    disp(strcat('pass',num2str(i),'Complete'));
end
    

% Display mean squared adaptation error
figure(1)
% Combine a cell array into one matrix
plot(cell2mat(neterror));
disp(strcat('Mean squared adaptation error=',num2str(mse(neterror))));

% Plot measured output and NN estimate
figure(2)
plot(1:N-2,t,'b',1:N-2,sim(net,P),'r');